@@ -41,6 +41,7 @@ use vars qw{
   $MIN_SPAM_CORPUS_SIZE_FOR_BAYES
   $MIN_HAM_CORPUS_SIZE_FOR_BAYES
   %HEADER_NAME_COMPRESSION
+  $OPPORTUNISTIC_LOCK_VALID
 };
 
 @ISA = qw();
@@ -153,6 +154,9 @@ use constant TOKENIZE_LONG_TOKENS_AS_SKIPS => 1;
 $MIN_SPAM_CORPUS_SIZE_FOR_BAYES = 200;
 $MIN_HAM_CORPUS_SIZE_FOR_BAYES = 200;
 
+# How many seconds should the opportunistic_expire lock be valid?
+$OPPORTUNISTIC_LOCK_VALID = 300;
+
 # Should we use the Robinson f(w) equation from
 # http://radio.weblogs.com/0101454/stories/2002/09/16/spamDetection.html ?
 # It gives better results, in that scores are more likely to distribute
@@ -199,6 +203,7 @@ sub new {
   my $self = {
     'main'              => $main,
     'log_raw_counts'	=> 0,
+    'conf'		=> $main->{conf},
 
     # Off. See comment above cached_probs_get().
     #'cached_probs'	=> { },
@@ -217,7 +222,7 @@ sub new {
 
 sub finish {
   my $self = shift;
-  if (!$self->{conf}->{use_bayes}) { return; }
+  if (!$self->{main}->{conf}->{use_bayes}) { return; }
   $self->{store}->untie_db();
 }
 
@@ -940,14 +945,7 @@ sub scan {
   $self->{store}->add_touches_to_journal();
   $self->{store}->scan_count_increment();
 
-  # handle expiry and journal syncing
-  if ($self->{store}->expiry_due()) {
-    dbg ("expiration is due: expiring old tokens now...");
-    $self->{store}->sync_journal();
-    $self->{store}->expire_old_tokens();
-    dbg ("expiration done");
-  }
-
+  $self->opportunistic_expire();
   $self->{store}->untie_db();
   return $score;
 
@@ -957,6 +955,19 @@ skip:
   return 0.5;           # nice and neutral
 }
 
+sub opportunistic_expire {
+  my($self) = @_;
+
+  # Is an expire or journal sync running?
+  my $running_expire = $self->{store}->get_running_expire_tok();
+  if ( defined $running_expire && $running_expire+$OPPORTUNISTIC_LOCK_VALID > time() ) { return; }
+
+  # handle expiry and journal syncing
+  if ($self->{store}->expiry_due()) {
+    $self->sync();
+  }
+}
+
 ###########################################################################
 
 sub dbg { Mail::SpamAssassin::dbg (@_); }
